﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SilenceORM.DbUtility
{
    /// <summary>
    /// 数据操作提供类
    /// </summary>
    /// <typeparam name="Connection"></typeparam>
    public class DbProvider<Connection> where Connection : DbConnection, new()
    {
        private static readonly int m_TimeOut = 600;
        private static readonly CommandType m_DefaultCmdType = CommandType.Text;
        private DbProviderFactory m_Factory;

        public DbProvider()
        {
            Connection instance = Activator.CreateInstance<Connection>();
            try
            {
                m_Factory = DbProviderFactories.GetFactory(instance);
            }
            finally
            {
                if (instance != null)
                    instance.Dispose();
            }
        }

        public DbConnection CreateConnection(string connStr)
        {
            DbConnection conn = m_Factory.CreateConnection();
            conn.ConnectionString = connStr;
            return conn;
        }

        public DbCommand CreateCommand(DbConnection conn)
        {
            return conn.CreateCommand();
        }

        public DbTransaction CreateTransaction(DbConnection conn)
        {
            return conn.BeginTransaction();
        }

        public DbParameter CreateParameter()
        {
            return m_Factory.CreateParameter();
        }

        public void PrepareCommand(DbConnection conn, DbCommand cmd)
        {
            PrepareCommand(conn, cmd, null, m_DefaultCmdType, string.Empty, null);
        }

        public void PrepareCommand(DbConnection conn, DbCommand cmd, DbTransaction tran)
        {
            PrepareCommand(conn, cmd, tran, m_DefaultCmdType, string.Empty, null);
        }

        public void PrepareCommand(DbConnection conn, DbCommand cmd, DbTransaction tran, CommandType cmdType, string commText, params DbParameter[] parms)
        {
            if (conn.State != ConnectionState.Open)
                conn.Open();
            cmd.Connection = conn;
            cmd.CommandTimeout = DbProvider<Connection>.m_TimeOut;
            cmd.CommandText = commText;
            cmd.CommandType = cmd.CommandType;
            cmd.Parameters.Clear();

            if (cmd.Transaction != null)
                cmd.Transaction = tran;

            if (parms == null || parms.Length == 0)
                return;

            foreach (DbParameter parm in parms)
            {
                if ((parm.Direction == ParameterDirection.Input || parm.Direction == ParameterDirection.InputOutput) && parm.Value == null)
                    parm.Value = DBNull.Value;
            }
            cmd.Parameters.AddRange(parms);
        }
    }
}
