﻿using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

using SilenceORM.DbUtility;
using SilenceORM.CommonUtility;

namespace SilenceORM.EF
{
    public class EFContext
    {
        public static IList<T> Select<T>(T condition)
            where T : new()
        {
            //sql语句、sql参数
            Tuple<string, DbParameter[], Dictionary<string, string>> sqlParameter = EFToSQL<T>.OperateSQL(condition, Operate.SELECT, "SELECT * FROM {0}");

            using (DbDataReader reader = SqlContext.GetInstance().ExecuteDataReader(sqlParameter.Item1, sqlParameter.Item2))
            {
                List<T> models = new List<T>();
                while (reader.Read())
                {
                    T t = new T();
                    foreach (string key in sqlParameter.Item3.Keys)
                    {
                        if (reader[sqlParameter.Item3[key]] != null)
                            t.GetType().InvokeMember(key, BindingFlags.SetProperty, null, t, new[] { reader[sqlParameter.Item3[key]] });
                    }
                    models.Add(t);
                }
                return models;
            }
        }

        public static int Insert<T>(T model) where T : new()
        {
            if (model == null)
                throw new ArgumentNullException(model.ToString());

            //sql语句、sql参数
            Tuple<string, DbParameter[], Dictionary<string, string>> sqlParameter = EFToSQL<T>.OperateSQL(model, Operate.INSERT, "INSERT INTO {0}");

            return SqlContext.GetInstance().ExecuteNonQuery(sqlParameter.Item1, sqlParameter.Item2);
        }

        public static int Update<T>(T model) where T : new()
        {
            if (model == null)
                throw new ArgumentNullException(model.ToString());

            //sql语句、sql参数
            Tuple<string, DbParameter[], Dictionary<string, string>> sqlParameter = EFToSQL<T>.OperateSQL(model, Operate.UPDATE, "UPDATE {0}");

            return SqlContext.GetInstance().ExecuteNonQuery(sqlParameter.Item1, sqlParameter.Item2);
        }

        public static int Delete<T>(T condition) where T : new()
        {
            if (condition == null)
                throw new ArgumentNullException(condition.ToString());

            //sql语句、sql参数
            Tuple<string, DbParameter[], Dictionary<string, string>> sqlParameter = EFToSQL<T>.OperateSQL(condition, Operate.DELETE, "DELETE FROM {0}");

            return SqlContext.GetInstance().ExecuteNonQuery(sqlParameter.Item1, sqlParameter.Item2);
        }
    }
}
