﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;

using SilenceORM.TableConfig;

namespace SilenceORM.CommonUtility
{
    public class ReflectorHelper
    {
        /// <summary>
        /// 得到类型所有实例属性（含特性）
        /// </summary>
        /// <typeparam name="T">类型</typeparam>
        /// <returns>键值对：Key：表特性；值：属性名-列特性</returns>
        public static KeyValuePair<TableAttribute, Dictionary<string, ColumnAttribute>> GetPropertiesForModel<T>() where T : new()
        {
            KeyValuePair<TableAttribute, Dictionary<string, ColumnAttribute>> KVProperties;

            Type type = typeof(T);

            //表
            TableAttribute tableAttr = type.GetCustomAttribute<TableAttribute>();
            if (tableAttr == null)
                tableAttr = new TableAttribute(name: type.Name);
            KVProperties = new KeyValuePair<TableAttribute, Dictionary<string, ColumnAttribute>>(tableAttr, new Dictionary<string, ColumnAttribute> { });

            //属性
            PropertyInfo[] properties = type.GetProperties(BindingFlags.Public | BindingFlags.Instance);
            foreach (PropertyInfo proInfo in properties)
            {
                //类直接过滤
                if (!proInfo.GetType().IsClass)
                    continue;

                ColumnAttribute attr = proInfo.GetCustomAttribute<ColumnAttribute>();
                KVProperties.Value.Add(proInfo.Name, attr ?? new ColumnAttribute(proInfo.Name));
            }

            return KVProperties;
        }
    }
}
